class Color:

    def __init__(self, R, G, B):
        self.red = R
        self.green = G
        self.blue = B

    def __str__(self):
        return '{0.red} {0.green} {0.blue}\n'.format(self)

    def hsv_to_rgb(H, S, V):
        h = H // 60
        f = H / 60 - h
        p = int(V*(1-S)*255)
        q = int(V*(1-S*f)*255)
        t = int(V*(1-S*(1-f))*255)
        V = int(V*255)
        if h in [0, 6]:
            return Color(V, t, p)
        elif h == 1:
            return Color(q, V, p)
        elif h == 2:
            return Color(p, V, t)
        elif h == 3:
            return Color(p, q, V)
        elif h == 4:
            return Color(t, p, V)
        elif h == 5:
            return Color(V, p, q)
        else:
            print('Fehler!')
            return None

    def heat_map_colors(n):
        palette = []
        for i in range(0, n+1):
            winkel = int(-240*i/n + 240)
            palette.append(Color.hsv_to_rgb(winkel, 1, 1))
        return palette



# Predefined colors
Color.White = Color(255, 255, 255)
Color.Black = Color(0, 0, 0)
Color.Red = Color(255, 0, 0)
Color.Green = Color(0, 128, 0) # darker
Color.Blue = Color(0, 0, 255)
Color.Cyan = Color(0, 255, 255)
Color.Magenta = Color(255, 0, 255)
Color.Yellow = Color(255, 255, 0)
    


# PPM = Portable PIxMap (einfaches Bitmap-Grafikformat mit 256^3 Farben)
class PPM:
    '''Klasse für S/W-Grafiken mit Speicherung im PPM-Format'''

    def __init__(self, width, height, bg=Color.White):
        '''Matrix (width x height) mit Hintergrundfarbe bg=0 (Weiss)'''
        self.w = width
        self.h = height
        self.img = [[bg for j in range(self.w)] for i in range(self.h)]

    def set_pixel(self, row, column, color=Color.Black):
        '''Setze Pixel auf den Wert color=1 (Schwarz)'''
        self.img[row][column] = color

    def write(self, filename):
        '''Schreibt Bild im PPM-Textformat in die Datei "filename"''' 
        fd = open(filename, mode='w')
        fd.write('P3\n') # PPM-Header
        fd.write('{0} {1}\n'.format(self.w, self.h)) # Bilddimension
        fd.write('255\n') # maximale Intensität
        for i in range(0, self.h):
            for j in range(0, self.w):
                fd.write('{0}'.format(self.img[i][j]))
        fd.close()

############################################################
# Folgender Code wird nur dann ausgeführt, wenn dieses
# Modul als erstes ('main') gestartet wird. Wird es
# hingegen von einem anderen Modul importiert, dann 
# ist die folgende Bedinung falsch und die darunter
# stehenden Zeilen werden 'ignoriert'.
############################################################
if __name__ == '__main__':

    def test(width, height, color):
        '''Erzeugt ein Bild und speichert es als "testbild.ppm"'''
        img = PPM(width, height)
        for i in range(0, height):
            for j in range(0, width):
                if j % 10 < 5:
                    img.set_pixel(row=i, column=j, color=color)
        img.write('testbild.ppm')

    def farbtonskala(width, height):
        '''Erzeugt eine Farbtonskala Bild und speichert es als "testbild.ppm"'''
        img = PPM(width, height)
        for i in range(0, height):
            for j in range(0, width):
                color = Color.hsv_to_rgb(int(j/width*360), 1, 1)
                img.set_pixel(row=i, column=j, color=color)
        img.write('farbtonskala.ppm')

    farbtonskala(800, 200)

    print(list(map(str, Color.heat_map_colors(5))))


        
    

